"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TcDate = void 0;
var tslib_1 = require("tslib");
var get_date_locale_1 = require("../../helpers/get-date-locale");
var date_fns_1 = require("date-fns");
var getISOWeekYear_1 = tslib_1.__importDefault(require("date-fns/getISOWeekYear"));
var getISOWeek_1 = tslib_1.__importDefault(require("date-fns/getISOWeek"));
var getISODay_1 = tslib_1.__importDefault(require("date-fns/getISODay"));
var mappers_1 = require("../../mappers");
var TcDate = /** @class */ (function () {
    function TcDate(date, format) {
        var _this = this;
        this.getWeekDates = function (weekStartDay) {
            var startOfWeekDate = date_fns_1.startOfWeek(_this.date, {
                weekStartsOn: weekStartDay,
            });
            var weekDates = [];
            for (var i = 0; i < 7; i++) {
                weekDates.push(date_fns_1.addDays(startOfWeekDate, i));
            }
            return weekDates;
        };
        this.getDayShortName = function (locale) {
            var dayShortName = _this.format('EEE d', locale);
            return dayShortName.charAt(0).toUpperCase() + dayShortName.slice(1);
        };
        this.getYearMonthDayKey = function () {
            return _this.format('yyyy-MM-dd');
        };
        this.getDateOfWeek = function (week, year) {
            var day = 1 + (week - 1) * 7;
            return new Date(year, 0, day);
        };
        this.getDateOfIsoWeek = function (week, year) {
            var simple = new Date(Date.UTC(year, 0, 1 + (week - 1) * 7));
            var dow = simple.getDay();
            var isoWeekStart = simple;
            if (dow <= 4) {
                isoWeekStart.setDate(simple.getDate() - simple.getDay() + 1);
            }
            else {
                isoWeekStart.setDate(simple.getDate() + 8 - simple.getDay());
            }
            return isoWeekStart;
        };
        this.getIsoWeekAndYear = function () {
            return {
                year: getISOWeekYear_1.default(_this.date),
                week: getISOWeek_1.default(_this.date),
                day: getISODay_1.default(_this.date),
            };
        };
        this.getISODay = function () {
            return getISODay_1.default(_this.date);
        };
        this.getISOYear = function () {
            return getISOWeekYear_1.default(_this.date);
        };
        this.getIsoWeek = function () {
            return getISOWeek_1.default(_this.date);
        };
        this.getDateWithoutTimeAsUTC = function () {
            if (!_this.date) {
                throw new Error('Date is null');
            }
            return new Date(Date.UTC(_this.date.getFullYear(), _this.date.getMonth(), _this.date.getDate(), 0, 0, 0, 0));
        };
        if (!date) {
            date = new Date();
        }
        if (typeof date === 'string') {
            if (format) {
                this.date = date_fns_1.parse(date, format, new Date());
            }
            else {
                this.date = date_fns_1.parseJSON(date);
            }
        }
        else {
            this.date = date;
        }
    }
    TcDate.prototype.toDate = function () {
        return this.date;
    };
    TcDate.prototype.format = function (template, locale) {
        if (locale === void 0) { locale = 'en'; }
        if (template === 'h:mma') {
            template = 'h:mmaaa';
        }
        var mappedTemplate = mappers_1.mapTcDateFormatToDateFnsDateFormat(template);
        return date_fns_1.format(this.date, mappedTemplate, {
            locale: get_date_locale_1.getDateLocale(locale),
        });
    };
    TcDate.prototype.add = function (amount, type) {
        var date;
        switch (type) {
            case 'milliseconds':
            case 'millisecond':
            case 'ms':
                date = date_fns_1.addMilliseconds(this.date, amount);
                break;
            case 'seconds':
            case 'second':
            case 's':
                date = date_fns_1.addSeconds(this.date, amount);
                break;
            case 'minutes':
            case 'minute':
            case 'm':
                date = date_fns_1.addMinutes(this.date, amount);
                break;
            case 'hours':
            case 'hour':
            case 'h':
                date = date_fns_1.addHours(this.date, amount);
                break;
            case 'days':
            case 'day':
            case 'd':
                date = date_fns_1.addDays(this.date, amount);
                break;
            case 'weeks':
            case 'week':
            case 'w':
                date = date_fns_1.addWeeks(this.date, amount);
                break;
            case 'months':
            case 'month':
            case 'M':
                date = date_fns_1.addMonths(this.date, amount);
                break;
            case 'quarters':
            case 'quarter':
            case 'q':
                date = date_fns_1.addQuarters(this.date, amount);
                break;
            case 'years':
            case 'year':
            case 'y':
                date = date_fns_1.addYears(this.date, amount);
                break;
            default:
                throw new Error(type + " is not supported");
        }
        return new TcDate(date);
    };
    TcDate.prototype.subtract = function (amount, type) {
        var date;
        switch (type) {
            case 'milliseconds':
            case 'millisecond':
            case 'ms':
                date = date_fns_1.subMilliseconds(this.date, amount);
                break;
            case 'seconds':
            case 'second':
            case 's':
                date = date_fns_1.subSeconds(this.date, amount);
                break;
            case 'minutes':
            case 'minute':
            case 'm':
                date = date_fns_1.subMinutes(this.date, amount);
                break;
            case 'hours':
            case 'hour':
            case 'h':
                date = date_fns_1.subHours(this.date, amount);
                break;
            case 'days':
            case 'day':
            case 'd':
                date = date_fns_1.subDays(this.date, amount);
                break;
            case 'weeks':
            case 'week':
            case 'w':
                date = date_fns_1.subWeeks(this.date, amount);
                break;
            case 'months':
            case 'month':
            case 'M':
                date = date_fns_1.subMonths(this.date, amount);
                break;
            case 'quarters':
            case 'quarter':
            case 'q':
                date = date_fns_1.subQuarters(this.date, amount);
                break;
            case 'years':
            case 'year':
            case 'y':
                date = date_fns_1.subYears(this.date, amount);
                break;
            default:
                throw new Error(type + " is not supported");
        }
        return new TcDate(date);
    };
    TcDate.prototype.diff = function (start, type) {
        var difference = date_fns_1.differenceInSeconds(this.date, start);
        switch (type) {
            case 'milliseconds':
            case 'millisecond':
            case 'ms':
                difference = date_fns_1.differenceInMilliseconds(this.date, start);
                break;
            case 'seconds':
            case 'second':
            case 's':
                difference = date_fns_1.differenceInSeconds(this.date, start);
                break;
            case 'minutes':
            case 'minute':
            case 'm':
                difference = date_fns_1.differenceInMinutes(this.date, start);
                break;
            case 'hours':
            case 'hour':
            case 'h':
                difference = date_fns_1.differenceInHours(this.date, start);
                break;
            case 'days':
            case 'day':
            case 'd':
                difference = date_fns_1.differenceInDays(this.date, start);
                break;
            case 'weeks':
            case 'week':
            case 'w':
                difference = date_fns_1.differenceInWeeks(this.date, start);
                break;
            case 'months':
            case 'month':
            case 'M':
                difference = date_fns_1.differenceInMonths(this.date, start);
                break;
            case 'quarters':
            case 'quarter':
            case 'q':
                difference = date_fns_1.differenceInQuarters(this.date, start);
                break;
            case 'years':
            case 'year':
            case 'y':
                difference = date_fns_1.differenceInYears(this.date, start);
                break;
            default:
                throw new Error(type + " is not supported");
        }
        return difference;
    };
    TcDate.prototype.startOf = function (type, weekStartDay) {
        var date;
        switch (type) {
            case 'seconds':
            case 'second':
            case 's':
                date = date_fns_1.startOfSecond(this.date);
                break;
            case 'minutes':
            case 'minute':
            case 'm':
                date = date_fns_1.startOfMinute(this.date);
                break;
            case 'hours':
            case 'hour':
            case 'h':
                date = date_fns_1.startOfHour(this.date);
                break;
            case 'days':
            case 'day':
            case 'd':
                date = date_fns_1.startOfDay(this.date);
                break;
            case 'weeks':
            case 'week':
            case 'w':
                date = date_fns_1.startOfWeek(this.date, { weekStartsOn: weekStartDay });
                break;
            case 'months':
            case 'month':
            case 'M':
                date = date_fns_1.startOfMonth(this.date);
                break;
            case 'quarters':
            case 'quarter':
            case 'q':
                date = date_fns_1.startOfQuarter(this.date);
                break;
            case 'years':
            case 'year':
            case 'y':
                date = date_fns_1.startOfYear(this.date);
                break;
            default:
                throw new Error(type + " is not supported");
        }
        return new TcDate(date);
    };
    TcDate.prototype.endOf = function (type, weekStartDay) {
        var date;
        switch (type) {
            case 'seconds':
            case 'second':
            case 's':
                date = date_fns_1.endOfSecond(this.date);
                break;
            case 'minutes':
            case 'minute':
            case 'm':
                date = date_fns_1.endOfMinute(this.date);
                break;
            case 'hours':
            case 'hour':
            case 'h':
                date = date_fns_1.endOfHour(this.date);
                break;
            case 'days':
            case 'day':
            case 'd':
                date = date_fns_1.endOfDay(this.date);
                break;
            case 'weeks':
            case 'week':
            case 'w':
                date = date_fns_1.endOfWeek(this.date, { weekStartsOn: weekStartDay });
                break;
            case 'months':
            case 'month':
            case 'M':
                date = date_fns_1.endOfMonth(this.date);
                break;
            case 'quarters':
            case 'quarter':
            case 'q':
                date = date_fns_1.endOfQuarter(this.date);
                break;
            case 'years':
            case 'year':
            case 'y':
                date = date_fns_1.endOfYear(this.date);
                break;
            default:
                throw new Error(type + " is not supported");
        }
        return new TcDate(date);
    };
    TcDate.prototype.weekday = function () {
        return date_fns_1.getDay(this.date);
    };
    TcDate.prototype.weekdayToDate = function (day, weekStartDay) {
        return date_fns_1.setDay(this.date, day, { weekStartsOn: weekStartDay });
    };
    TcDate.prototype.week = function (weekStartDay) {
        return date_fns_1.getWeek(this.date, { weekStartsOn: weekStartDay });
    };
    TcDate.prototype.year = function () {
        return date_fns_1.getYear(this.date);
    };
    TcDate.prototype.isSame = function (dateToCheck, type, weekStartDay) {
        switch (type) {
            case 'days':
            case 'day':
            case 'd':
                return date_fns_1.isSameDay(this.date, dateToCheck);
            case 'weeks':
            case 'week':
            case 'w':
                return date_fns_1.isSameWeek(this.date, dateToCheck, {
                    weekStartsOn: weekStartDay,
                });
            case 'months':
            case 'month':
            case 'M':
                return date_fns_1.isSameMonth(this.date, dateToCheck);
            case 'quarters':
            case 'quarter':
            case 'q':
                return date_fns_1.isSameQuarter(this.date, dateToCheck);
            case 'years':
            case 'year':
            case 'y':
                return date_fns_1.isSameYear(this.date, dateToCheck);
            default:
                throw new Error(type + " is not supported");
        }
    };
    TcDate.prototype.isBetween = function (_a) {
        var start = _a.start, end = _a.end;
        return date_fns_1.isWithinInterval(this.date, {
            start: start,
            end: end,
        });
    };
    TcDate.prototype.isAfter = function (dateToCheck) {
        return date_fns_1.isAfter(this.date, dateToCheck);
    };
    TcDate.prototype.isValid = function () {
        return date_fns_1.isValid(this.date);
    };
    TcDate.prototype.set = function (amount, type) {
        var date;
        switch (type) {
            case 'second':
            case 'seconds':
                date = date_fns_1.setSeconds(this.date, amount);
                break;
            case 'minute':
            case 'minutes':
                date = date_fns_1.setMinutes(this.date, amount);
                break;
            case 'hour':
            case 'hours':
                date = date_fns_1.setHours(this.date, amount);
        }
        return new TcDate(date);
    };
    TcDate.prototype.daysInMonth = function () {
        return date_fns_1.getDaysInMonth(this.date);
    };
    TcDate.prototype.toISOString = function () {
        return this.date.toISOString();
    };
    TcDate.prototype.interval = function (start, end) {
        return date_fns_1.eachDayOfInterval({
            start: start,
            end: end,
        });
    };
    return TcDate;
}());
exports.TcDate = TcDate;
//# sourceMappingURL=date.js.map