var TimeParseRule = /** @class */ (function () {
    function TimeParseRule(regExRule, value, extractionStrategy) {
        this.regExRule = regExRule;
        this.value = value;
        this.secondsExtractionStrategy = extractionStrategy;
    }
    TimeParseRule.prototype.parseRule = function () {
        var result = this.regExRule.exec(this.value);
        if (result === null || result.length === 0) {
            return null;
        }
        var seconds = this.secondsExtractionStrategy(result);
        if (seconds === null) {
            return null;
        }
        return { seconds: seconds };
    };
    return TimeParseRule;
}());
var SECONDS_IN_HOUR = 3600;
var MINUTES_IN_HOUR = 60;
var SECONDS_IN_MINUTE = 60;
var HOURS_IN_DAY = 24;
var HOURS_IN_12_CLOCK = 12;
var SECONDS_IN_DAY = HOURS_IN_DAY * SECONDS_IN_HOUR;
var SIGN_MINUS = -1;
var SIGN_PLUS = 1;
var unifySeparator = function (value) { return value.replace(/[,;]/giu, '.'); };
var extractSign = function (result) {
    return result.groups['sign'] ? SIGN_MINUS : SIGN_PLUS;
};
var extractHours = function (result) {
    return parseFloat(unifySeparator(result.groups['hours']));
};
var extractMinutes = function (result) {
    return parseInt(result.groups['minutes'], 10);
};
var extractAbbreviation = function (result) { var _a; return (_a = result.groups['abbreviation']) === null || _a === void 0 ? void 0 : _a.toLowerCase(); };
// Main strategies
function fromHoursSigned(result) {
    var sign = extractSign(result);
    var hours = extractHours(result);
    if (hours > HOURS_IN_DAY) {
        return null;
    }
    var correctedSign = hours === 0 ? SIGN_PLUS : sign; // edge case: zero cannot have minus mark
    return correctedSign * hours * SECONDS_IN_HOUR;
}
function fromMinutesSigned(result) {
    var sign = extractSign(result);
    var minutes = extractMinutes(result);
    if (minutes > HOURS_IN_DAY * MINUTES_IN_HOUR) {
        return null;
    }
    var correctedSign = minutes === 0 ? SIGN_PLUS : sign; // edge case: zero cannot have minus mark
    return correctedSign * minutes * SECONDS_IN_MINUTE;
}
function fromHoursAndMinutesSigned(result) {
    var sign = extractSign(result);
    var hours = extractHours(result);
    var minutes = extractMinutes(result);
    var totalSeconds = hours * SECONDS_IN_HOUR + minutes * SECONDS_IN_MINUTE;
    if (hours > HOURS_IN_DAY ||
        minutes > MINUTES_IN_HOUR - 1 ||
        totalSeconds > SECONDS_IN_DAY) {
        return null;
    }
    var correctedSign = totalSeconds === 0 ? SIGN_PLUS : sign; // edge case: zero cannot have minus mark
    return correctedSign * totalSeconds;
}
function from12HoursClock(result) {
    var hours = extractHours(result);
    var minutes = extractMinutes(result);
    var abbreviation = extractAbbreviation(result);
    if (hours > HOURS_IN_12_CLOCK || minutes > MINUTES_IN_HOUR - 1) {
        return null;
    }
    if (abbreviation === 'pm') {
        hours = hours + HOURS_IN_12_CLOCK;
    }
    var totalSeconds = hours * SECONDS_IN_HOUR + minutes * SECONDS_IN_MINUTE;
    return totalSeconds;
}
var TimeParser = /** @class */ (function () {
    function TimeParser(value) {
        this.value = value;
    }
    TimeParser.prototype.parse = function () {
        if (!this.value) {
            return {
                isValid: true,
                seconds: 0,
            };
        }
        var result = this.parseRules(this.value);
        return {
            isValid: result !== null,
            seconds: result,
        };
    };
    TimeParser.prototype.parseRules = function (value) {
        var rules = [
            // 12 hours clock, am/pm
            new TimeParseRule(/^(?<hours>\d{1,2}):(?<minutes>\d{2})(?<abbreviation>[ap]m)$/giu, value, from12HoursClock),
            // positive or negative 1-digit or 2-digit hours
            new TimeParseRule(/^(?<sign>-)?(?<hours>\d{1,2})h$/giu, value, fromHoursSigned),
            // positive or negative 1-digit or 2-digit or 3-digit minutes
            new TimeParseRule(/^(?<sign>-)?(?<minutes>\d{1,3})m$/giu, value, fromMinutesSigned),
            // positive or negative 1-digit or 2-digit hours and 1-digit or 2-digit minutes
            new TimeParseRule(/^(?<sign>-)?(?<hours>\d{1,2})h(?<minutes>\d{1,2})m$/giu, value, fromHoursAndMinutesSigned),
            // 1-digit
            new TimeParseRule(/^(?<sign>-)?(?<hours>\d)$/giu, value, fromHoursSigned),
            // 2 digits, first digit is not a zero
            new TimeParseRule(/^(?<sign>-)?(?<hours>[1-9]\d)$/giu, value, fromHoursSigned),
            // 2-digits, first digit is a zero
            new TimeParseRule(/^(?<sign>-)?(?<minutes>0\d)$/giu, value, fromMinutesSigned),
            // 3-digit
            new TimeParseRule(/^(?<sign>-)?(?<hours>\d)(?<minutes>\d\d)$/giu, value, fromHoursAndMinutesSigned),
            // 4-digit
            new TimeParseRule(/^(?<sign>-)?(?<hours>\d\d)(?<minutes>\d\d)$/giu, value, fromHoursAndMinutesSigned),
            // digits with separator
            new TimeParseRule(/^(?<sign>-)?(?<hours>\d\d)[,.;:](?<minutes>\d\d)$/giu, value, fromHoursAndMinutesSigned),
            // 2 digits with separator
            new TimeParseRule(/^(?<hours>\d{1,2}):(?<minutes>\d{2})$/giu, value, fromHoursAndMinutesSigned),
            // Scenario for single digits with separator
            new TimeParseRule(/^(?<sign>-)?(?<hours>\d[,.;]\d)$/giu, value, fromHoursSigned),
            // digits with separator without second value
            new TimeParseRule(/^(?<sign>-)?(?<hours>\d\d)[,.;:]$/giu, value, fromHoursSigned),
            // single digit with separator without second value
            new TimeParseRule(/^(?<sign>-)?(?<hours>\d)[,.;:]$/giu, value, fromHoursSigned),
        ];
        for (var i = 0; i < rules.length; i++) {
            var parseResult = rules[i].parseRule();
            if (parseResult) {
                return parseResult.seconds;
            }
        }
        return null;
    };
    return TimeParser;
}());
export { TimeParser };
//# sourceMappingURL=time-parser.js.map